//
// Filename     : baseCompartmentNeighborhood.cc
// Description  : A base class describing variable updates
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2003
// Revision     : $Id: baseCompartmentNeighborhood.cc 474 2010-08-30 15:23:20Z henrik $
//

#include<vector>


#include"baseCompartmentNeighborhood.h"
#include "../reactions/elongatedBacteria/NeighborhoodLatticeWalls2D.h"
#include "../reactions/sphericalBacteria/NeighborhoodLatticeWalls2D.h"
#include "../reactions/sphericalBacteria/NeighborhoodLattice.h"
#include"compartmentNeighborhood.h"

BaseCompartmentNeighborhood::~BaseCompartmentNeighborhood(){}

BaseCompartmentNeighborhood *
BaseCompartmentNeighborhood::
createCompartmentNeighborhood(std::vector<double> &paraValue,
															std::vector< std::vector<size_t> > &indValue, 
															const std::string &idValue ) {
  
  // Distance rules
  //(compartmentNeighborhood.h,compartmentNeighborhood.cc)
  if( idValue=="neighborhoodLatticeCigar" ) 
    return new NeighborhoodLatticeCigar(paraValue,indValue);
  else if( idValue=="neighborhoodLatticeCigarMultipleWall" ) 
      return new NeighborhoodLatticeCigarMultipleWall(paraValue,indValue);
	else if( idValue=="elongatedBacteria::neighborhoodLatticeWalls2D" ) 
		return new elongatedBacteria::NeighborhoodLatticeWalls2D(paraValue,indValue);
	else if( idValue=="sphericalBacteria::neighborhoodLatticeWalls2D" ) 
		return new sphericalBacteria::NeighborhoodLatticeWalls2D(paraValue,indValue);
	else if( idValue=="sphericalBacteria::neighborhoodLattice2D" ) 
		return new sphericalBacteria::NeighborhoodLattice<2>(paraValue,indValue);
	else if( idValue=="sphericalBacteria::neighborhoodLattice3D" ) 
		return new sphericalBacteria::NeighborhoodLattice<3>(paraValue,indValue);
  else if( idValue=="neighborhoodDistanceSphere" ) 
    return new NeighborhoodDistanceSphere(paraValue,indValue);
  else if( idValue=="neighborhoodDistanceSpherePeriodic" ) 
    return new NeighborhoodDistanceSpherePeriodic(paraValue,indValue);
  else if( idValue=="neighborhoodDistanceSphereWithWalls" ) 
    return new NeighborhoodDistanceSphereWithWalls(paraValue,indValue);
  else if( idValue=="neighborhoodDistanceSphereBud" )
    return new NeighborhoodDistanceSphereBud(paraValue,indValue);
  else if( idValue=="neighborhoodDistanceEllipse" )
    return new NeighborhoodDistanceEllipse(paraValue,indValue);
  else if( idValue=="neighborhoodDistanceCigar" )
    return new NeighborhoodDistanceCigar(paraValue,indValue);
	else if (idValue == "neighborhoodDistanceSpherePeriodicBox")
		return new NeighborhoodDistanceSpherePeriodicBox(paraValue, indValue);

	// From indices
	// (compartmentNeighborhood.h, compartmentNeighborhood.cc)
	else if (idValue == "neighborhoodIndex")
		return new NeighborhoodIndex(paraValue, indValue);
        // Null
	else if (idValue == "nullNeighborhood")
		return new NullNeighborhood(paraValue, indValue);

	
  // From file
  //(compartmentNeighborhood.h,compartmentNeighborhood.cc)
  else if( idValue=="neighborhoodFromFileInitial" ) {
    std::cerr << "\nBaseCompartmentNeighborhood::"
	      << "createCompartmentNeighborhood(pVal,indVal,idVal) "
	      << "Should never find idValue=" << idValue << "\n";
    exit(-1);
  }
  //Default, if nothing found
  else {
    std::cerr << "\nBaseCompartmentNeighborhood::"
	      << "createCompartmentNeighborhood() "
	      << "WARNING: CompartmentNeighborhoodtype " << idValue 
	      << " not known, no compartmentNeighborhood created.\n\7";
    exit(-1);
  }
}

BaseCompartmentNeighborhood * 
BaseCompartmentNeighborhood::
createCompartmentNeighborhood(std::istream &IN ) {
  
 
  std::string idVal;
  size_t pNum,levelNum;
  IN >> idVal;
  
  if( idVal == "neighborhoodFromFileInitial" ) {
    IN >> pNum;
    std::vector<double> pVal( pNum );
    for( size_t i=0 ; i<pNum ; i++ )
      IN >> pVal[i];
    std::string fileName;
    IN >> fileName;
    return new NeighborhoodFromFileInitial(pVal,fileName);
  }
  IN >> pNum;
  IN >> levelNum;
  std::vector<size_t> varIndexNum( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    IN >> varIndexNum[i];
  
  std::vector<double> pVal( pNum );
  for( size_t i=0 ; i<pNum ; i++ )
    IN >> pVal[i];
  
  std::vector< std::vector<size_t> > varIndexVal( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    varIndexVal[i].resize( varIndexNum[i] );
  
  for( size_t i=0 ; i<levelNum ; i++ )
    for( size_t j=0 ; j<varIndexNum[i] ; j++ )
      IN >> varIndexVal[i][j];
	
  return createCompartmentNeighborhood(pVal,varIndexVal,idVal);
}

unsigned int BaseCompartmentNeighborhood::
create(Organism &O,std::vector< std::vector<double> > &y,double t ) {
  std::cerr << "BaseCompartmentNeighborhood::create() should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}  

unsigned int BaseCompartmentNeighborhood::
update(Organism &O,std::vector< std::vector<double> > &y,double t ) {
  std::cerr << "BaseCompartmentNeighborhood::update() should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}  
