 

#ifndef BASEESTIMATOR_H
#define BASEESTIMATOR_H

#include <iostream>
#include <fstream>

#include "../solvers/baseSolver.h"
#include "baseProblem.h"

///
///@brief A base class using a factory method to create optimizers. 
///

typedef int ID_t;

class BaseEstimator : public BaseOptimizationProblem {
	
 private:
  BaseSolver *S_;
  size_t numPara_;						// Total number of parameters (Caught from the solver)
  size_t  numCostFile_, numInitFile_;				// should be the same, by the way
  
  std::vector< std::vector< std::vector<double> > > init_;	// Triple table of num files * numCompartments * numVariables
  
  std::vector<std::string> costTemplateFile_, initFile_;	
  
  std::vector< std::vector<size_t> > costLists_;
  std::vector< std::vector<double> > costTemplateTime_;
  std::vector< std::vector<std::vector<std::vector<double> > > > costTemplate_;
  
  static BaseEstimator *createEstimator(BaseSolver *S, const std::string &file);
  static BaseEstimator* createEstimator(BaseSolver *S, const std::ifstream &IN);
  void readEstimator(const std::string &file);
  void readEstimator(std::ifstream &IN); // The reading point should be just  before NumCostFiles (ie very beginning of the cost file)
  
  void readCostTemplate();
  void readInit();
  
  //std::vector<bool> table_occupation_ID;
  //std::vector< std::vector< std::vector< std::vector<double> > > > saved_init_;
  
 public:
  BaseEstimator(BaseSolver *S, const std::string &file);
  BaseEstimator(BaseSolver *S, const std::ifstream &IN);
  virtual ~BaseEstimator();
  double getCost();
  void printState(double E,std::ostream &os = std::cout) const;
  
  //void set_init_from_ID(ID_t);
  //void save_values_to_ID(ID_t);
  //void kill_ID(ID_t);
  //ID_t create_ID();
  //void set_initial_init();
 private:
  void save_initial_init_to_ID0();
  
  
  inline size_t numPara() const;
  inline double parameter(size_t i) const;
  inline void setParameter(size_t i, double value);
  //inline void setParameter(std::vector<double> &parameter);    
  
};

inline size_t BaseEstimator::numPara() const
{
  return numPara_;
}

inline double BaseEstimator::parameter(size_t i) const
{
  return S_->getOrganism()->parameter(i);
}    

inline void BaseEstimator::setParameter(size_t i, double value) 
{
  S_->getOrganism()->setParameter(i, value);
}

#endif		
