//
// Filename     : costFunction.cc
// Description  : Classes defining cost functions
// Author(s)    : Pontus Melke (pontus@thep.lu.se)
// Created      : August 2006
// Revision     : $Id: costFunction.cc 474 2010-08-30 15:23:20Z henrik $
//

#include "costFunction.h"
#include<cstdlib>

double MeanSquare::getCost(double simValue, double dataValue, double scaleFactor)
{
	//std::cerr << "caclulating cost to: ";
	double costFactor= scaleFactor*(simValue-dataValue);
	//std::cerr << costFactor*costFactor << "\n";
	return costFactor*costFactor;
}

double MeanSquareRelative::getCost(double simValue, double dataValue, double scaleFactor)
{
	double costFactor= scaleFactor*(simValue-dataValue)/dataValue;	
	return costFactor*costFactor;
}

double NormalizedMeanSquare::getCost(double simValue, double dataValue, double scaleFactor)
{
  double costFactor= scaleFactor*(simValue-dataValue)/dataValue;	
  return costFactor*costFactor;
}

double NormalizedMeanSquare::
addCost(const std::vector< std::vector<double> > &y, const double t) 
{ 
  if( !costList_.size() ) {
    std::cerr << "BaseCostFunction::addCost() No variables marked for "
	      << "cost calculation. Nothing added to cost.\n";
    return 0.;
  }
  if( costTemplateIndex_<0 || costTemplateIndex_>=costTemplateTime_.size() ) {
    std::cerr << "BaseCostFunction::addCost() Wrong template index for "
	      << "cost calculation. Nothing added to cost.\n";
    return 0.;    
  }  
  if( costTemplate_[costTemplateIndex_].size() != y.size() ) {
    std::cerr << "BaseCostFunction::addCost() Not correct number of cells "
	      << "to compare simulation with cost template\n";
    exit(-1);
  }
  if( std::fabs( t-costTemplateTime_[costTemplateIndex_] ) > 0.001 ) {
    std::cerr << "BaseCostFunction::addCost() Warning: time difference "
	      << "between simulation and cost template (" 
	      << t << " " << costTemplateTime_[costTemplateIndex_] << "  "
	      << "costTemplateIndex: " << costTemplateIndex_ << " "
	      << std::fabs( t-costTemplateTime_[costTemplateIndex_] )
	      << ")\n";
    exit(-1);
  }
  double costAdd=0.;
  double costAddPrev=0.0;
  size_t xI=0,yI=1;
  for (size_t k=0 ; k<costList_.size() ; ++k ) {
    int j=costList_[k];
    for (size_t i=0 ; i<y.size() ; ++i ) {
      double normFactor=1.0;
      if (k==0) {//WUSCHEL
	if (y[i][xI]<1 && y[i][yI]<-1)
	  normFactor=78;
	else
	  normFactor=0.1;
      }
      else if (k==1) {//KANADI
      }
      else if (k==2) {//CLAVATA3
      }
      else {
	std::cerr << "NormalizedMeanSquare::addCost() Only defined for clvWusKan...\n";
	exit(-1);
      }
      costAdd += getCost(y[i][j],costTemplate_[costTemplateIndex_][i][k],normFactor);
    } 
    //		costIndividual_[costTemplateIndex_][k] = (costAdd-costAddPrev)/(2*y.size()*costList_.size()*costTemplate_.size());
    costAddPrev = costAdd;
  }
  cost_ += costAdd/(2*y.size()*costList_.size()*costTemplate_.size());
  costTemplateIndex_++;
  return costAdd;
}

