/**
 * Filename     : embedding.cc
 * Description  : A class describing a embedding (position updates)
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: embedding.cc 474 2010-08-30 15:23:20Z henrik $
 */

#include<iostream>
#include<fstream>
#include "embedding.h"

Embedding::Embedding() {
  //std::cerr << "Embedding::Embedding() not defined.\n";
}

Embedding::Embedding( const Embedding & embeddingCopy ) { 
  
  setId( embeddingCopy.id() );
  setDimension( embeddingCopy.dimension() );
  setReaction( embeddingCopy.reaction() );
}

Embedding::Embedding( char *inFile ) {
  
  std::ifstream IN( inFile );
  if( !IN ) {
    std::cerr << "Embedding::Embedding() - "
	      << "Cannot open file " << inFile << "\n\n\7";exit(-1);}  
  readEmbedding(IN);
}

Embedding::Embedding( const std::string &inFile ) { 
  
  const char *tmp = inFile.c_str();
  std::ifstream IN( tmp );
  if( !IN ) {
    std::cerr << "Embedding::Embedding() - "
	      << "Cannot open file " << inFile << "\n\n\7";exit(-1);}
  readEmbedding(IN);
}

Embedding::Embedding( std::ifstream &IN ) {
  
  readEmbedding(IN);
}

Embedding::~Embedding() {
  //std::cerr << "Embedding::~Embedding() not defined.\n";
}
  
//!Read an embedding from an open file 
void Embedding::readEmbedding( std::ifstream &IN ) {
  
  //Remove all reactions before adding
  if( numReaction() )
    reaction_.resize(0);
  
  int dimensionVal,numReactionVal;
  std::string idVal;
  IN >> idVal;
  setId(idVal);
  IN >> dimensionVal;
  setDimension(dimensionVal);
  IN >> numReactionVal;
  for( int i=0 ; i<numReactionVal ; i++ )
    if( addReaction(IN) )
      std::cerr << "Embedding::readEmbedding() Warning Adding reaction failed "
		<< "for embedding " << id() << "\n";
}

//!Adds a reaction to the list from an open file
int Embedding::addReaction( std::ifstream &IN ) {
  if( !IN ) return -1;
  reaction_.push_back( SimpleReaction::createReaction(IN) );
  return 0;
}

//!Adds up all derivative contributions from the reactions
void Embedding::derivs(Compartment &compartment,
		       std::vector< std::vector<double> > &y,
		       std::vector< std::vector<double> > &dydt ) {
  
  for( int i=0 ; i<numReaction() ; i++ )
    reaction(i)->derivs(compartment,compartment.embeddingStart(),y,dydt);
}
