/**
 * Filename     : printModel.cc
 * Description  : Printing an Organism model 
 *              : (can be used to check the model file)
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : February 2004
 * Revision     : $Id: printModel.cc 474 2010-08-30 15:23:20Z henrik $
 */
#include<vector>
#include<iostream>
#include"../../organism.h"
#include"../../solvers/rk5Adaptive.h"

int main(int argc,char *argv[]) {
  
  //Input handeling
  //////////////////////////////////////////////////////////////////////
  if( argc != 3 ) {
    std::cerr << "usage: " << argv[0] << " modelFile initFile\n";
    exit(-1);
  }
  std::string modelFile=argv[1];
  std::string initFile=argv[2];
  
  //Create and print the organism (model)
  //////////////////////////////////////////////////////////////////////
  Organism O(modelFile,initFile);

  O.printParameter();

  if( O.numNeighborhood() ) {
    double t=0.0;
    std::vector< std::vector<double> > y(O.numCompartment());
    for( size_t i=0 ; i<O.numCompartment() ; i++ )
      y[i] = O.compartment(i).variable();
    O.neighborhoodCreate(y,t);
  }
  
  std::cerr << "\nVariables:\n";
  O.printVariable();
  
  if( O.numNeighborhood() ) {
    std::cerr << "\nNeighborhood:\n";
    O.printNeighbor();
  }
}
