/**
 * Filename     : simulatorRk4RelaxMechanics.cc
 * Description  : Simulator where the mechanics is relaxed at each time
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : May 2006
 * Revision     : $Id: simulatorRk4RelaxMechanics.cc 474 2010-08-30 15:23:20Z henrik $
 */
#include<vector>
#include<iostream>
#include<fstream>
#include"../../organism.h"
#include"../../solvers/rk4.h"

int main(int argc,char *argv[]) {
  
  //Input handeling
  //////////////////////////////////////////////////////////////////////
  if( argc != 5 ) {
    std::cerr << "usage: " << argv[0] << " modelFile initFile "
	      << "simulatorParaFile relaxParaFile\n";
    exit(-1);
  }
  std::string modelFile=argv[1];
  std::string initFile=argv[2];
  std::string simPara=argv[3];
  std::string relaxPara=argv[4];
  
  //Define the organism (model) and simulator
  //////////////////////////////////////////////////////////////////////
  Organism O(modelFile,initFile);
  Rk4 S(&O,simPara);
  //O.printParameter();
  
  //Open relaxPara file and read relax variables and reactions
  //////////////////////////////////////////////////////////////////////
  std::vector<size_t> relaxReaction,relaxPos;
  size_t numPos,numReaction;
  double cEps=1e-3;
  double cTmax=10000;
  std::ifstream IN(argv[4]);
  if( !IN ) {
    std::cerr << "Cannot open relaxation parameter file.\n";
    exit(0);
  }
  IN >> numPos;
  IN >> numReaction;
  relaxPos.resize(numPos);
  relaxReaction.resize(numReaction);
  for( size_t i=0 ; i<relaxPos.size() ; ++i )
    IN >> relaxPos[i];
  for( size_t i=0 ; i<relaxReaction.size() ; ++i )
    IN >> relaxReaction[i];
  IN >> cEps;
  IN >> cTmax;
  IN.close();

  //Simulate with relaxed mechanics
  //////////////////////////////////////////////////////////////////////
  std::cerr << "Start simulation." << std::endl;
  std::cerr << "Relaxing " << relaxPos.size() << " positional variables "
	    << " using " << relaxReaction.size() << " reactions with "
	    << "maxDydt=" << cEps << " and maxT=" << cTmax << std::endl;
  
  S.simulateWithConvergingMechanics(cEps,cTmax,relaxReaction,relaxPos);
}
