/**
 * Filename     : simulatorStaticNeighborhood.cc
 * Description  : Organism simulation without updating neighborhood
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : February 2004
 * Revision     : $Id: simulatorStaticNeighborhood.cc 474 2010-08-30 15:23:20Z henrik $
 */
#include<vector>
#include<iostream>
#include"../../organism.h"
#include"../../solvers/rk5Adaptive.h"

int main(int argc,char *argv[]) {
  
  //Input handeling
  //////////////////////////////////////////////////////////////////////
  if( argc != 4 ) {
    std::cerr << "usage: " << argv[0] << " modelFile initFile "
	      << "simulatorParaFile\n";
    exit(-1);
  }
  std::string modelFile=argv[1];
  std::string initFile=argv[2];
  std::string simPara=argv[3];
  
  //Define the organism and simulator
  //////////////////////////////////////////////////////////////////////
  Organism O(modelFile,initFile);
  Rk5Adaptive S(&O,simPara);
  //O.printParameter();
  
  //Copy the y matrix x,y,r and calculate neighborhood
  //This is done once here and never during the simulation.
  //////////////////////////////////////////////////////////////////////
  int newNumCol=3;
  std::vector< std::vector<double> > y1,y1Tmp;
  y1Tmp = S.y();
  y1.resize( y1Tmp.size() );
  for( int i=0 ; i<y1.size() ; i++ ) {
    y1[i].resize( newNumCol );
    for( int j=0 ; j<y1[i].size() ; j++ )
      y1[i][j] = y1Tmp[i][j];
  }
  O.findNeighbors(y1);
  
  //Simulate without updating neighborhood
  //////////////////////////////////////////////////////////////////////
  S.simulateStaticNeighborhood();
}
