//
// Filename     : baseReaction.cc
// Description  : A base class describing variable updates
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2003
// Revision     : $Id: baseReaction.cc 499 2011-09-15 12:56:59Z henrik $
//

#include <vector>

#include "../organism.h"
#include "baseReaction.h"

#include "adhocReaction.h"
#include "completeModel.h"
#include "creation.h"
#include "degradation.h"
#include "grn.h"
#include "growth.h"
#include "massAction.h"
#include "mechanical.h"
#include "transport.h"
#include "rootTransport.h"
#include "rootTransport2D.h"
#include "meshedRoot.h"
#include "elongatedBacteria.h"
#include "sphericalBacteria.h"
#include "simplifiedAuxinTransportJTB/auxinTransport.h"
#include "clawus/clawusDynamics.h"
#include "clawus/clvcrn.h"
#include "BenchMarkMendez.h"

BaseReaction::~BaseReaction()
{
}

BaseReaction *
BaseReaction::createReaction(std::vector<double> &paraValue,
			     std::vector< std::vector<size_t> > &indValue, 
			     const std::string &idValue ) 
{
	std::cerr << idValue << "\n";
	//Grn type of (creation) reactions
	//grn.h,grn.cc
	if( idValue=="grn" ) 
		return new Grn(paraValue,indValue);
	else if( idValue=="gsrn" )
		return new Gsrn(paraValue,indValue);
	else if(idValue=="gsrn2")
		return new Gsrn2(paraValue,indValue);
	else if(idValue=="michaelisMenten")
		return new MichaelisMenten(paraValue,indValue);  
	else if(idValue=="michaelisMentenRestricted")
		return new MichaelisMentenRestricted(paraValue,indValue);  
	else if(idValue=="hill")
		return new Hill(paraValue,indValue);  
	else if(idValue=="hillRepressor")
		return new HillRepressor(paraValue,indValue);  
	else if(idValue=="sheaAckers")
		return new SheaAckers(paraValue,indValue);  
	else if(idValue=="sheaAckersDimer")
		return new SheaAckersDimer(paraValue,indValue);  
	//Creation
	//creation.h,creation.cc
	else if(idValue=="creationLinear")
		return new CreationLinear(paraValue,indValue);
	else if(idValue=="creationZero")
		return new CreationZero(paraValue,indValue);
	else if(idValue=="creationOne")
		return new CreationOne(paraValue,indValue);
	else if(idValue=="creationTwo")
		return new CreationTwo(paraValue,indValue);
	else if(idValue=="creationSpatialThreshold")
		return new CreationSpatialThreshold(paraValue,indValue);
	else if(idValue=="creationSpatialSphere")
		return new CreationSpatialSphere(paraValue,indValue);
	else if(idValue=="creationSpatialSphereCylinder")
		return new CreationSpatialSphereCylinder(paraValue,indValue);
	else if(idValue=="creationSpatialMax")
		return new CreationSpatialMax(paraValue,indValue);
	else if(idValue=="creationExponent")
		return new CreationExponent(paraValue,indValue);
	else if(idValue=="creationZeroDegradationOne")
		return new CreationZeroDegradationOne(paraValue,indValue);
	else if(idValue=="creationZeroDegradationTwo")
		return new CreationZeroDegradationTwo(paraValue,indValue);
	else if(idValue=="creationOneDegradationOne")
		return new CreationOneDegradationOne(paraValue,indValue);
	else if(idValue=="creationOneDegradationTwo")
		return new CreationOneDegradationTwo(paraValue,indValue);
	else if(idValue=="constant")
		return new Constant(paraValue,indValue);
	else if(idValue=="creationNeighbor")
		return new CreationNeighbor(paraValue,indValue);
	//Degradation
	//degradation.h,degradation.cc
	else if(idValue=="degradationLinear")
		return new DegradationLinear(paraValue,indValue);
	else if(idValue=="degradationOne")
		return new DegradationOne(paraValue,indValue);
	else if(idValue=="degradationTwo")
		return new DegradationTwo(paraValue,indValue);
	else if(idValue=="degradationThree")
		return new DegradationThree(paraValue,indValue);
	else if(idValue=="degradationHill")
		return new DegradationHill(paraValue,indValue);
	else if(idValue=="degradationHillR")
		return new DegradationHillR(paraValue,indValue);
	else if(idValue=="degradationSpatialThreshold")
	  return new DegradationSpatialThreshold(paraValue,indValue);
	//Mass action types of reactions
	//massAction.h,massAction.cc
	else if( idValue=="massAction" )
		return new MassAction(paraValue,indValue);
	else if( idValue=="massAction1to1" )
		return new MassAction1to1(paraValue,indValue);
	else if( idValue=="massAction1to2" )
		return new MassAction1to2(paraValue,indValue);
	else if( idValue=="massAction1to3" )
		return new MassAction1to3(paraValue,indValue);
	else if( idValue=="massAction1to4" )
		return new MassAction1to4(paraValue,indValue);
	else if( idValue=="massAction2to1" )
		return new MassAction2to1(paraValue,indValue);
	else if( idValue=="massAction3to1" )
		return new MassAction3to1(paraValue,indValue);
	else if( idValue=="massActionEnzymatic" )
		return new MassActionEnzymatic(paraValue,indValue);
	else if( idValue=="massActionMM" )
		return new MassActionMM(paraValue,indValue);
	else if( idValue=="massActionEnzymaticMM" )
		return new MassActionEnzymaticMM(paraValue,indValue);
	else if( idValue=="massActionMMFraction" )
		return new MassActionMMFraction(paraValue,indValue);  
	//Transport
	//transport/diffusion.h
	else if(idValue=="diffusion")
		return new Diffusion(paraValue,indValue);
	else if(idValue=="diffusionSimple")
		return new DiffusionSimple(paraValue,indValue);
	else if(idValue=="diffusionSimpleSpatial")
		return new DiffusionSimpleSpatial(paraValue,indValue);
	else if(idValue=="diffusionRestricted")
		return new DiffusionRestricted(paraValue,indValue);
	else if(idValue=="diffusionSimpleRestricted")
		return new DiffusionSimpleRestricted(paraValue,indValue);
	//transport/polarizationTransport.h
	else if(idValue=="polarizationTransport")
		return new PolarizationTransport(paraValue,indValue);
	else if(idValue=="polarizationTransportFast")
		return new PolarizationTransportFast(paraValue,indValue);
	else if(idValue=="polarizationTransportFastMM")
		return new PolarizationTransportFastMM(paraValue,indValue);
	else if(idValue=="symmetricTransportFastMM")
		return new SymmetricTransportFastMM(paraValue,indValue);
	else if(idValue=="polarizationTransportInfluxFastMM")
		return new PolarizationTransportInfluxFastMM(paraValue,indValue);
	else if(idValue=="polarizationHillTransportInfluxFastMM")
		return new PolarizationHillTransportInfluxFastMM(paraValue,indValue);
	else if(idValue=="symmetricTransportInfluxFastMM")
		return new SymmetricTransportInfluxFastMM(paraValue,indValue);
	else if(idValue=="polarizationTransportFastMMSpatial")
		return new PolarizationTransportFastMMSpatial(paraValue,indValue);
	else if(idValue=="polarizationHillTransportFast")
		return new PolarizationHillTransportFast(paraValue,indValue);
	else if(idValue=="polarizationHillTransportFastMM")
		return new PolarizationHillTransportFastMM(paraValue,indValue);
	else if(idValue=="polarizationExpTransportFastMM")
		return new PolarizationExpTransportFastMM(paraValue,indValue);
	else if(idValue=="polarizationConstLinTransportFast")
		return new PolarizationConstLinTransportFast(paraValue,indValue);
	else if(idValue=="polarizationTransportFastB")
		return new PolarizationTransportFastB(paraValue,indValue);
	else if(idValue=="polarizationTransportFastBMM")
		return new PolarizationTransportFastBMM(paraValue,indValue);
	else if(idValue=="polarizationTransportFastSwitchedMM")
		return new PolarizationTransportFastSwitchedMM(paraValue,indValue);
	else if(idValue=="polarizationLinFastTransportRestrictedMM")
		return new PolarizationLinFastTransportRestrictedMM(paraValue,indValue);
	else if(idValue=="polarizationConstFastTransportRestrictedMM")
		return new PolarizationConstFastTransportRestrictedMM(paraValue,indValue);
	else if(idValue=="polarizationConstFastTransportFractionRestrictedMM")
		return new PolarizationConstFastTransportFractionRestrictedMM(paraValue,indValue);
	else if(idValue=="polarizationConstHillFastTransportFractionRestrictedMM")
		return new PolarizationConstHillFastTransportFractionRestrictedMM(paraValue,indValue);
	else if(idValue=="polarizationConstHillFastTransportRestrictedMM")
		return new PolarizationConstHillFastTransportRestrictedMM(paraValue,indValue);
	else if(idValue=="polarizationConstHillFastTransportRestrictedMM")
		return new PolarizationConstHillFastTransportRestrictedMM(paraValue,indValue);
	else if(idValue=="polarizationTransportFastGeneralFGRestricted")
		return new PolarizationTransportFastGeneralFGRestricted(paraValue,indValue);
	else if(idValue=="generalPolarizationTransportFast")
		return new GeneralPolarizationTransportFast(paraValue,indValue);
	//transport/directionalTransport.h
	else if(idValue=="directionalTransport")
		return new DirectionalTransport(paraValue,indValue);
	else if(idValue=="directionalTransportMM")
		return new DirectionalTransportMM(paraValue,indValue);
	else if(idValue=="directionalTransportSwitchedMM")
		return new DirectionalTransportSwitchedMM(paraValue,indValue);
	//transport/membraneTransport.h
	else if(idValue=="activeTransportRestricted")
		return new ActiveTransportRestricted(paraValue,indValue);
	else if(idValue=="activeTransportMMRestricted")
		return new ActiveTransportMMRestricted(paraValue,indValue);
	else if(idValue=="activeTransportBRestricted")
		return new ActiveTransportBRestricted(paraValue,indValue);
	else if(idValue=="membraneTransport")
		return new MembraneTransport(paraValue,indValue);
	else if(idValue=="membraneTransportRestricted")
		return new MembraneTransportRestricted(paraValue,indValue);
	else if(idValue=="activeMembraneTransportRestricted")
		return new ActiveMembraneTransportRestricted(paraValue,indValue);
	else if(idValue=="activeMembraneTransportRestrictedMM")
		return new ActiveMembraneTransportRestrictedMM(paraValue,indValue);
	else if(idValue=="activeMembraneTransportBRestricted")
		return new ActiveMembraneTransportBRestricted(paraValue,indValue);
	else if(idValue=="activeMembraneTransportBHillRestricted")
		return new ActiveMembraneTransportBHillRestricted(paraValue,indValue);
	else if(idValue=="activeTransportVariableSimple")
		return new ActiveTransportVariableSimple(paraValue,indValue);
	//rootTransport.h
	else if(idValue=="rootTransport")
		return new RootTransport(paraValue,indValue);
	else if(idValue=="rootTransport2D")
		return new RootTransport2D(paraValue,indValue);
	//meshedRoot/*.h
	else if(idValue=="meshedRoot::activeTransport")
		return new meshedRoot::ActiveTransport(paraValue,indValue);
	else if(idValue=="meshedRoot::activeTransportMM")
		return new meshedRoot::ActiveTransportMM(paraValue,indValue);
	else if(idValue=="meshedRoot::membraneTransportRestricted")
		return new meshedRoot::MembraneTransportRestricted(paraValue,indValue);
	else if(idValue=="meshedRoot::diffusionRestricted")
	  return new meshedRoot::DiffusionRestricted(paraValue,indValue);
	else if(idValue=="meshedRoot::outflux")
	  return new meshedRoot::Outflux(paraValue,indValue);
	else if(idValue=="meshedRoot::influx")
	  return new meshedRoot::Influx(paraValue,indValue);
	else if(idValue=="meshedRoot::outfluxMM")
	  return new meshedRoot::OutfluxMM(paraValue,indValue);
	else if(idValue=="meshedRoot::influxMM")
		return new meshedRoot::InfluxMM(paraValue,indValue);
	else if(idValue=="meshedRoot::constantMassProduction")
		return new meshedRoot::ConstantMassProduction(paraValue,indValue);
	//Growth related updates
	//growth.h,growth.cc
	else if(idValue=="growthConstant")
		return new GrowthConstant(paraValue,indValue);
	else if(idValue=="growthExponential")
		return new GrowthExponential(paraValue,indValue);
	else if(idValue=="growthExponentialOne")
		return new GrowthExponentialOne(paraValue,indValue);
	else if(idValue=="growthExponentialRestricted")
		return new GrowthExponentialRestricted(paraValue,indValue);
	else if(idValue=="growthExponentialTrunc")
		return new GrowthExponentialTrunc(paraValue,indValue);
	else if(idValue=="growthExponentialRestrictedTrunc")
		return new GrowthExponentialRestrictedTrunc(paraValue,indValue);
	else if(idValue=="dilution")
		return new Dilution(paraValue,indValue);
	else if(idValue=="ellipticGrowthExponentialTrunc")
		return new EllipticGrowthExponentialTrunc(paraValue,indValue);
	else if(idValue=="cigarGrowthExponentialTrunc")
		return new CigarGrowthExponentialTrunc(paraValue,indValue);
	else if(idValue=="sphereBudGrowthTruncNeigh")
		return new SphereBudGrowthTruncNeigh(paraValue,indValue);
	else if(idValue=="lensRadiusGrowth")
		return new LensRadiusGrowth(paraValue,indValue);
	else if(idValue=="lensLayerGrowth")
		return new LensLayerGrowth(paraValue,indValue);
	else if(idValue=="lensDilution")
		return new LensDilution(paraValue,indValue);
	//Mechanical interactions between compartments
	//mechanical.h,mechanical.cc
	else if(idValue=="spring")
		return new Spring(paraValue,indValue);
	else if(idValue=="springAsymmetric")
		return new SpringAsymmetric(paraValue,indValue);
	else if(idValue=="springTruncated")
		return new SpringTruncated(paraValue,indValue); 
	else if(idValue=="springAdhesionRestrictedSphere")
		return new SpringAdhesionRestrictedSphere(paraValue,indValue); 
	else if(idValue=="springAsymmetricSphere")
		return new SpringAsymmetricSphere(paraValue,indValue);
	else if(idValue=="springAsymmetricCylinder")
		return new SpringAsymmetricCylinder(paraValue,indValue);
	else if(idValue=="springAsymmetricSphereCylinder")
		return new SpringAsymmetricSphereCylinder(paraValue,indValue);
	else if(idValue=="sphereCylinderWall")
		return new SphereCylinderWall(paraValue,indValue);
	else if(idValue=="ellipticRepulsion")
		return new EllipticRepulsion(paraValue,indValue);
	else if(idValue=="normalEllipticRepulsion")
		return new NormalEllipticRepulsion(paraValue,indValue);
	else if(idValue=="springAsymmetricEllipse")
		return new SpringAsymmetricEllipse(paraValue,indValue);
	else if(idValue=="springAsymmetricSphereBud")
		return new SpringAsymmetricSphereBud(paraValue,indValue);
	else if(idValue=="wallSpringElliptic")
		return new WallSpringElliptic(paraValue,indValue);
	else if(idValue=="wallSpringCigar")
		return new WallSpringCigar(paraValue,indValue);
	else if(idValue=="springElasticWallEllipse")
		return new SpringElasticWallEllipse(paraValue,indValue);
	else if(idValue=="gravitationElliptic")
		return new GravitationElliptic(paraValue,indValue);
	else if(idValue=="springAsymmetricCigar")
		return new SpringAsymmetricCigar(paraValue,indValue);
	else if(idValue=="springRepulsiveCigar")
		return new SpringRepulsiveCigar(paraValue,indValue);
	else if(idValue=="springWallCigar2")
		return new SpringWallCigar2(paraValue,indValue);
	else if(idValue=="springElasticWallCigar2")
		return new SpringElasticWallCigar2(paraValue,indValue);
	else if(idValue=="lagrangianCigar")
		return new LagrangianCigar(paraValue,indValue);
	else if(idValue=="lagrangianWallCigar")
		return new LagrangianWallCigar(paraValue,indValue);
	else if(idValue=="perforatedLagrangianWallCigar")
		return new PerforatedLagrangianWallCigar(paraValue,indValue);
	else if(idValue=="lagrangianCigarNew")
		return new LagrangianCigarNew(paraValue,indValue);
	else if(idValue=="lagrangianCigarFriction")
		return new LagrangianCigarFriction(paraValue,indValue);
	else if(idValue=="lagrangianWallCigarNew")
		return new LagrangianWallCigarNew(paraValue,indValue);
	else if(idValue=="multipleWallCigar")
		return new MultipleWallCigar(paraValue,indValue);
	else if(idValue=="cigarAdhesion")
		return new CigarAdhesion(paraValue,indValue);
	else if(idValue=="wallCigar3D")
		return new WallCigar3D(paraValue,indValue);	
	// mechanics/sphereForce
	else if(idValue=="sphere::ForceDirection")
	  return new Sphere::ForceDirection(paraValue,indValue);	
	//TODO else if(idValue=="sphere::ForceRestricted")
	//  return new Sphere::ForceRestricted(paraValue,indValue);	
	// mechanics/sphereWall
	else if(idValue=="sphere::sphericalWall")
	  return new Sphere::SphericalWall(paraValue,indValue);	
	// elongatedBacteria/
	else if(idValue=="elongatedBacteria::flagellarMotor") 
		return new elongatedBacteria::FlagellarMotor(paraValue,indValue);
	else if(idValue=="elongatedBacteria::cellMechanics2D") 
		return new elongatedBacteria::CellMechanics<2>(paraValue,indValue);
	else if(idValue=="elongatedBacteria::cellMechanics3D") 
		return new elongatedBacteria::CellMechanics<3>(paraValue,indValue);
	else if(idValue=="elongatedBacteria::cellMechanicsAdhesion2D") 
		return new elongatedBacteria::CellMechanicsAdhesion<2>(paraValue,indValue);
	else if(idValue=="elongatedBacteria::cellMechanicsAdhesion3D") 
		return new elongatedBacteria::CellMechanicsAdhesion<3>(paraValue,indValue);
	else if(idValue=="elongatedBacteria::wallMechanics2D") 
		return new elongatedBacteria::WallMechanics2D(paraValue,indValue);
	else if(idValue=="elongatedBacteria::wallMechanics3D") 
		return new elongatedBacteria::WallMechanics3D(paraValue,indValue);
	else if(idValue=="elongatedBacteria::exponentialGrowth") 
	 	return new elongatedBacteria::ExponentialGrowth(paraValue,indValue);
	else if(idValue=="elongatedBacteria::substrateDependentGrowth2D") 
		return new elongatedBacteria::SubstrateDependentGrowth2D(paraValue,indValue);
	else if(idValue=="elongatedBacteria::substrateDependentGrowth3D") 
		return new elongatedBacteria::SubstrateDependentGrowth3D(paraValue,indValue);
	else if(idValue=="elongatedBacteria::chemotaxisSimple")
		return new elongatedBacteria::ChemotaxisSimple(paraValue,indValue);
	//gradients
	else if(idValue=="elongatedBacteria::gradientSmooth2D")
		return new elongatedBacteria::GradientSmooth2D(paraValue,indValue);
	else if(idValue=="elongatedBacteria::gradientGaussian2D")
		return new elongatedBacteria::GradientGaussian2D(paraValue,indValue);
	else if(idValue=="sphericalBacteria::substrateDependentGrowth2D") 
		return new sphericalBacteria::SubstrateDependentGrowth2D(paraValue,indValue);
	else if(idValue=="sphericalBacteria::substrateDependentGrowth3D") 
		return new sphericalBacteria::SubstrateDependentGrowth3D(paraValue,indValue);
	else if(idValue=="sphericalBacteria::cellMechanics2D") 
		return new sphericalBacteria::CellMechanics<2>(paraValue,indValue);
	else if(idValue=="sphericalBacteria::cellMechanics3D") 
		return new sphericalBacteria::CellMechanics<3>(paraValue,indValue);
	else if(idValue=="sphericalBacteria::wallMechanics2D") 
		return new sphericalBacteria::WallMechanics2D(paraValue,indValue);
	//gradients
	else if(idValue=="sphericalBacteria::gradientSmooth2D")
		return new sphericalBacteria::GradientSmooth2D(paraValue,indValue);
	else if(idValue=="sphericalBacteria::gradientGaussian2D")
		return new sphericalBacteria::GradientGaussian2D(paraValue,indValue);
	//Complete models
	else if(idValue=="auxinFlux")
		return new AuxinFlux(paraValue,indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::auxinTransport")
		return new SimplifiedAuxinTransportJTB::AuxinTransport(paraValue, indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::auxinTransportExperimentalOne")
		return new SimplifiedAuxinTransportJTB::AuxinTransportExperimentalOne(paraValue, indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::auxinTransportExperimentalTwo")
		return new SimplifiedAuxinTransportJTB::AuxinTransportExperimentalTwo(paraValue, indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::auxinTransportPosterA")
		return new SimplifiedAuxinTransportJTB::AuxinTransportPosterA(paraValue, indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::auxinTransportPosterB")
		return new SimplifiedAuxinTransportJTB::AuxinTransportPosterB(paraValue, indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::auxinTransportPosterC")
		return new SimplifiedAuxinTransportJTB::AuxinTransportPosterC(paraValue, indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::auxinTransportNoCompetition")
		return new SimplifiedAuxinTransportJTB::AuxinTransportNoCompetition(paraValue, indValue);
	else if (idValue == "simplifiedAuxinTransportJTB::CWII")
		return new SimplifiedAuxinTransportJTB::CWII(paraValue, indValue);
	else if (idValue == "cellCellAuxinTransport")
		return new CellCellAuxinTransport(paraValue, indValue);
	else if (idValue == "cellWallAuxinTransport")
		return new CellWallAuxinTransport(paraValue, indValue);
	else if (idValue == "cytokininNetwork")
		return new CytokininNetwork(paraValue, indValue);
	// clawus/
	else if (idValue == "clawus::clawusDynamics")
	  return new clawus::ClawusDynamics(paraValue, indValue);
	else if (idValue == "clawus::clvcrn")
	  return new clawus::Clvcrn(paraValue, indValue);
	else if (idValue == "clawus::clvcrnNonLin")
	  return new clawus::ClvcrnNonLin(paraValue, indValue);
	// Ad-hoc reations adhocReaction.h
	else if (idValue == "sumCompartmentMass")
	  return new SumCompartmentMass(paraValue, indValue);
	else if (idValue == "diffusionDivided")
	  return new DiffusionDivided(paraValue, indValue);
	else if (idValue == "add")
	  return new Add(paraValue, indValue);
	else if (idValue == "switchCompartmentThreshold")
	  return new SwitchCompartmentThreshold(paraValue, indValue);
	//  BenchMarkMendex
	else if (idValue == "benchMarkMendez")
	  return new benchMarkMendez(paraValue, indValue);
	
	//Default, if nothing found
	else {
		std::cerr << "\nBaseReaction::createReaction() WARNING: Reactiontype " 
			  << idValue << " not known, no reaction created.\n\7";
		exit(-1);
	}
}

BaseReaction* 
BaseReaction::createReaction(std::istream &IN ) 
{
	std::string idVal;
  size_t pNum,levelNum;
  IN >> idVal;
  IN >> pNum;
  IN >> levelNum;
  std::vector<size_t> varIndexNum( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    IN >> varIndexNum[i];
  
  std::vector<double> pVal( pNum );
  for( size_t i=0 ; i<pNum ; i++ )
    IN >> pVal[i];
  
  std::vector< std::vector<size_t> > varIndexVal( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    varIndexVal[i].resize( varIndexNum[i] );
  
  for( size_t i=0 ; i<levelNum ; i++ )
    for( size_t j=0 ; j<varIndexNum[i] ; j++ )
      IN >> varIndexVal[i][j];
  
  return createReaction(pVal,varIndexVal,idVal);
}

void BaseReaction::
derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt) 
{
  std::cerr << "BaseReaction::derivs() should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}  

void BaseReaction::update(double h, double t,DataMatrix &y) 
{
}

size_t BaseReaction::
Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A)
{
  std::cerr << "BaseReaction::Jacobian() should not be used. "
	    << "Should always be mapped onto one of the real types." << std::endl
	    << "This indicates that the Jacobian is not defined for one of the reactions" << std::endl
	    << " and that an implicit solver might not be used." << std::endl;
  exit(0);
}

double BaseReaction::
propensity(Compartment &compartment,size_t species,DataMatrix &y)
{
  std::cerr << "BaseReaction::propensity() should not be used. "
	    << "Should always be mapped onto one of the real types." 
	    << std::endl
	    << "This indicates that the propensity is not defined for one of the reactions" 
	    << std::endl
	    << " and that a stochastic solver can not be used." << std::endl;
  exit(0);
}

void BaseReaction::
discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
{
  std::cerr << "BaseReaction::discreteUpdate() should not be used. "
	    << "Should always be mapped onto one of the real types." 
	    << std::endl
	    << "This indicates that the propensity is not defined for one of the reactions" 
	    << std::endl
	    << " and that a stochastic solver can not be used." << std::endl;
  exit(0);
}

void BaseReaction::print( std::ofstream &os ) 
{
  std::cerr << "BaseReaction::print(ofstream) should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}

Organism* BaseReaction::organism() const 
{
  return organism_;
}

/*size_t BaseReaction::numSpecies(){
	return organism()->numSpecies();
}*/
