import WCK
import os

#################
# Visualisation #
#################


# WCK.sam.test is used to display the simulation results in newman.
# each use will launch newman with a color scale going from 0 to 2 for variables concentrations (as presented in most of the Figures), and then with a color scale displaying the whole span of variable concentrations.

# Examples:
os.popen("mkdir test") 
WCK.sam.directory = "test" # Sets directory to "test/". Default directory for files is "default/"

### simulates the wild type behavior with example parameters, simulation will run for 200 time units (duration) and will be displayed in 100 time steps (steps).
# WCK.example is the set of parameters examplified throughout of the study
### WCK.example can be replaced by any list of 17 floats. The file parameters contains the 229 sets of parameters used in the study (17floats on each line).
#WCK.sam.test(["wild"], WCK.example, duration = 200, steps = 100)

### simulates the loss of function scenario for WUS (Wlof), and the pCLV3::WUS mutant (pCW), using the example parameters by first stabilizing the WT
#WCK.sam.test(["Wlof","pCW"], WCK.example, duration = 200, steps = 100)

### simulates the growth scenario
#WCK.sam.test(["growth"], WCK.example, duration = 300, steps = 100)

### The test function accepts:
## wild - for wild type
## Wlof, Clof, Klof - for loss of function
## Wover, Cover, Kover - for overexpression
## Wubi, Cubi, Kubi - for ubiquitous expression
## pWC, pWK, pCW, pCK, pKW, pKC - for mis-expression
## growth - for the growth simulation
### if a list of perturbations is given for test, the display order will be as in the 5 above lines.


################
# Optimization #
################

# Examples:

#os.popen("mkdir optimization") 
#WCK.sam.directory = "optimization"
#import random
#for i in range(0,5):
#    parameters = [random.random()*0.1 for i in range(0,17)]
#    WCK.sam.optimize(parameters,iters=10000,s2s = True)

### Starts 5 runs of optimization, initiated with random parameters. Each optimization stops after 10000 evaluations of the cost function.
### Result is stored in the optimization file. Each line of the file contains the parameters followed by the contribution of WUS, CLV3 and KAN1 to the cost function, and finally the value of the cost function for the considered set of parameters. Note that since it is a stochastic optimization algorithm it will not produce the same parameters at individual runs.


############################
# Perturbations evaluation #
############################

### Takes a file containing parameter sets and corresponding cost value (such as the "parameters" file included or any optimization file) and computes the variation between the stabilized wild type and the given perturbations.

# Example:

# os.popen("mkdir lof_mutants") 
# WCK.sam.directory = "lof_mutants"
# WCK.sam.compute_mutants(["Wlof","Clof","Klof"],"parameters",cost_limit=60)
# Computes the loss of function perturbations for the parameters contained in the file "parameter"

# os.popen("mkdir ubi_mutants") 
# WCK.sam.directory = "ubi_mutants"
# WCK.sam.compute_mutants(["Wubi","Cubi","Kubi"],"optimization/optimization",cost_limit=50)
## Computes the ubiquitous perturbation for a hypothetic optimization file ("optimization/optimization"). Computes only sets of parameters having a cost value below 50.

### Result file ("mutants") contains the parameter sets, and for each given perturbation, the variation of WUS, CLV3 and KAN1
